/*
 * Copyright (C) 2025 Raphael Mudge, Adversary Fan Fiction Writers Guild
 *
 * This file is part of Tradecraft Garden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, see <https://www.gnu.org/licenses/>.
 */

#include <windows.h>

typedef struct {
	__typeof__(LoadLibraryA)   * LoadLibraryA;
	__typeof__(GetProcAddress) * GetProcAddress;
} IMPORTFUNCS;

WINUSERAPI int WINAPI USER32$MessageBoxA(HWND hWnd,LPCSTR lpText,LPCSTR lpCaption,UINT uType);

/*
 * our xorkey, we're going to set this via our loader.spec
 */
char xorkey[128] = { 1 };

/*
 * A simple routine to obfuscate and de-obfuscate our data
 */
void applyxor(char * data, DWORD len) {
	for (DWORD x = 0; x < len; x++) {
		data[x] ^= xorkey[x % 128];
	}
}

/* globals to keep track of our DLL in memory. For simplicity's sake, this example
 * assumes the whole thing is RWX, but we could really do whatever we need between
 * the loader and this hooking module */
char * g_dllBase;
DWORD  g_dllSize;

/*
 * Our hooked functions that do what we want...
 */
int WINAPI _MessageBoxA(HWND hWnd,LPCSTR lpText,LPCSTR lpCaption,UINT uType) {
	int result;

	applyxor(g_dllBase, g_dllSize);

	// may as well use our own strings, because the originals are garbled right now
	result = USER32$MessageBoxA(hWnd, "Hello from hook.c!", "HOOKED!", uType);

	applyxor(g_dllBase, g_dllSize);

	return result;
}

/*
 * This is the GetProcAddress our loader will use. It's a chance for us to hook things. Notice, we also use
 * this to replace any other references to GetProcAddress with our hook function too. In this way, for the
 * things our DLL does... everything will use this GetProcAddress giving us nice visibility into everything.
 */
char * WINAPI _GetProcAddress(HMODULE hModule, LPCSTR lpProcName) {
	char * result = (char *)GetProcAddress(hModule, lpProcName);

	if ((char *)GetProcAddress == result) {
		return (char *)_GetProcAddress;
	}
	/* I like this method of identifying hook targets, because there are no strings to fuss with. BUT,
	 * beware... LIBRARY$func WILL LoadLibrary the specified library, here USER32.dll when the PICO is
	 * loaded into memory. So, if that's not desireable, you may need to switch to using string hashes or
	 * something else to decide what to hook */
	else if ((char *)USER32$MessageBoxA == result) {
		return (char *)_MessageBoxA;
	}

	return result;
}

/* Our entry point and our initialization for everything fun that will happen... */
void go(IMPORTFUNCS * funcs, char * dllBase, DWORD dllsz) {
	/* update the GetProcAddress our loader uses */
	funcs->GetProcAddress = (__typeof__(GetProcAddress) *)_GetProcAddress;

	/* track this information, because we will need it later */
	g_dllBase = dllBase;
	g_dllSize = dllsz;
}
