/*
 * Copyright (C) 2025 Raphael Mudge, Adversary Fan Fiction Writers Guild
 *
 * This file is part of Tradecraft Garden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, see <https://www.gnu.org/licenses/>.
 */

#include <windows.h>

/*
 * Set the return address and frame address of the (real) stack frame we want to point back to. I use
 * this to create a stack that looks like our non-module backed code was never called. If you don't have
 * anything like that, then set both to 0 to break some means of stack unwinding.
 */
typedef struct {
	ULONG_PTR   frameaddr;
	ULONG_PTR   retaddr;
} FRAME;

/*
 * This is an internal struct, just keeping some info off of the stack, so I don't have to keep
 * adjusting proxy.c based on the number of local vars.
 *
 * You, as an API caller, have no responsibility to any of this.
 */
typedef struct {
	FRAME     * p_frame;
	FRAME       frame;
} __INTERNAL;

/*
 * Our proxycall struct, this is where we pass in arguments for our PIC proxy function
 * to act on. Each of the arguments is annotated below.
 */
typedef struct {
	ULONG_PTR  function; 	/* a pointer to the function we want the proxy to call */
	DWORD      argc;	/* the number of arguments being passed, args[argc] and argc must agree */
	__INTERNAL temp;	/* don't touch this, not even to zero it, used by the proxy function */
	FRAME      spoofme;	/* the return address and frame address we want to "spoof" */
	ULONG_PTR  args[4];	/* the arguments to our function */
} PROXYCALL;

/*
 * And, as simple as this, it's our proxy call function
 */
typedef ULONG_PTR (*PROXY)(PROXYCALL * desc);
